#include "stdafx.h"
#include "FileListTabItem.h"
#include "../ConfigCode/ConfigFileListWindow.h"
#include "../resource.h"
#include "../Utilities/StringUtil.h"

CFileListTabItem::CFileListTabItem(CConfigManager &rMan):
	Model(rMan),
	ListView(rMan,Model),
	TreeView(Model),
	hMutex(NULL)
{
}

bool CFileListTabItem::CreateListView(HWND hParentWnd,HWND hFrameWnd,const CConfigFileListWindow &ConfFLW)
{
	//--t@CꗗEBhE쐬
	ListView.Create(hParentWnd,CWindow::rcDefault,NULL,WS_CHILD | /*WS_VISIBLE | */WS_CLIPSIBLINGS | WS_CLIPCHILDREN | LVS_OWNERDATA|LVS_AUTOARRANGE|LVS_SHAREIMAGELISTS|LVS_SHOWSELALWAYS, WS_EX_CLIENTEDGE);
	//t[EBhẼnh
	ListView.SetFrameWnd(hFrameWnd);

	//X^Cݒ
	ListView.SetExtendedListViewStyle(LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT | LVS_EX_GRIDLINES|LVS_EX_HEADERDRAGDROP);

	//Xgr[ɃJǉ
	if(!ListView.SetColumnState(ConfFLW.ColumnOrderArray))return false;

	//\ݒ
	ListView.SetDisplayFileSizeInByte(BOOL2bool(ConfFLW.DisplayFileSizeInByte));
	//\[gݒ
	if(ConfFLW.StoreSetting){
		Model.SetSortKeyType(ConfFLW.SortColumn);
		Model.SetSortMode(0!=ConfFLW.SortDescending);
	}else{
		Model.SetSortKeyType(-1);
		Model.SetSortMode(true);
	}

	//Xgr[X^C̐ݒ
	if(ConfFLW.StoreSetting){
		DWORD Style=ListView.GetWindowLong(GWL_STYLE);
		Style&=~(LVS_ICON|LVS_REPORT|LVS_SMALLICON|LVS_LIST);
		ListView.SetWindowLong(GWL_STYLE,Style|ConfFLW.ListStyle);
	}else{
		DWORD Style=ListView.GetWindowLong(GWL_STYLE);
		Style&=~(LVS_ICON|LVS_REPORT|LVS_SMALLICON|LVS_LIST);
		ListView.SetWindowLong(GWL_STYLE,Style|LVS_ICON);
	}
	return true;
}

bool CFileListTabItem::CreateTreeView(HWND hParentWnd,HWND hFrameWnd,const CConfigFileListWindow &ConfFLW)
{
	//c[r[쐬
	TreeView.Create(hParentWnd,CWindow::rcDefault,NULL,WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN|TVS_HASBUTTONS | TVS_HASLINES | TVS_LINESATROOT | TVS_SHOWSELALWAYS,WS_EX_CLIENTEDGE);

	//t[EBhẼnh
	TreeView.SetFrameWnd(hFrameWnd);
	return true;
}

bool CFileListTabItem::CreateTabItem(HWND hParentWnd,HWND hFrameWnd,const CConfigFileListWindow &ConfFLW)
{
	m_hFrameWnd=hFrameWnd;

	// Xvb^EBhE쐬
	CRect rc;
	GetClientRect(hParentWnd,rc);
	Splitter.Create(hParentWnd,rc,NULL,WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS);
	// Xvb^EBhEgX^Cݒ
	Splitter.SetSplitterExtendedStyle(0);

	//---c[r[
	if(!CreateTreeView(Splitter,hFrameWnd,ConfFLW))return false;
	//---Xgr[
	if(!CreateListView(Splitter,hFrameWnd,ConfFLW))return false;

	//EBhEɐݒ
	Splitter.SetSplitterPanes(TreeView,ListView);
	// o[̈ʒuݒ
	Splitter.SetSplitterPos(ConfFLW.TreeWidth);
	Splitter.UpdateSplitterLayout();
	return true;
}


HRESULT CFileListTabItem::OpenArchive(LPCTSTR lpszArc,DLL_ID forceID,const CConfigFileListWindow& ConfFLW,FILELISTMODE flMode,IArchiveContentUpdateHandler* lpHandler,CString &strErr)
{
	//---
	idForceDLL=forceID;
	HRESULT hr=Model.OpenArchiveFile(lpszArc,forceID,flMode,strErr,lpHandler);

	if(SUCCEEDED(hr)){
		//c[\z
		TreeView.ConstructTree();
		if(ConfFLW.ExpandTree)TreeView.ExpandTree();
	}
	return hr;
}

void CFileListTabItem::DestroyWindow()
{
	if(TreeView.IsWindow())TreeView.DestroyWindow();
	if(ListView.IsWindow())ListView.DestroyWindow();
	if(Splitter.IsWindow())Splitter.DestroyWindow();

	if(hMutex){
		CloseHandle(hMutex);
		hMutex=NULL;
	}
}

void CFileListTabItem::ShowWindow(int nCmdShow)
{
	TreeView.ShowWindow(nCmdShow);
	ListView.ShowWindow(nCmdShow);
	Splitter.ShowWindow(nCmdShow);
}

void CFileListTabItem::OnActivated()
{
	//---t[EBhECxgXiɓo^
	Model.addEventListener(m_hFrameWnd);
}

void CFileListTabItem::OnDeactivated()
{
	//---t[EBhECxgXi
	Model.removeEventListener(m_hFrameWnd);
}

DWORD CFileListTabItem::GetListViewStyle()
{
	return ListView.GetWindowLong(GWL_STYLE);
}

void CFileListTabItem::SetListViewStyle(DWORD dwStyleNew)
{
	DWORD dwStyle=ListView.GetWindowLong(GWL_STYLE);
	dwStyle&=~(LVS_ICON|LVS_REPORT|LVS_SMALLICON|LVS_LIST);
	ListView.SetWindowLong(GWL_STYLE,dwStyle|dwStyleNew);
}

