#include "stdafx.h"
#include "compress.h"
#include "Dialogs/LogDialog.h"

#include "resource.h"

#include "archivermanager.h"
#include "Utilities/Semaphore.h"
#include "Utilities/StringUtil.h"
#include "Utilities/FileOperation.h"
#include "Utilities/OSUtil.h"
#include "ConfigCode/ConfigManager.h"
#include "ConfigCode/ConfigCompress.h"
#include "ConfigCode/ConfigGeneral.h"
#include "CommonUtil.h"

bool Compress(std::list<CString> &ParamList,const PARAMETER_TYPE Type,CConfigManager &ConfigManager,int Options,LPCTSTR lpszFormat,LPCTSTR lpszMethod,LPCTSTR lpszLevel,LPCTSTR lpszSpecificOutputFileName,LPCTSTR lpszSpecificOutputDir)
{
	TRACE(_T("Function ::Compress() started.\n"));

	CArchiverDLLManager &ArchiverManager=CArchiverDLLManager::GetInstance();
	CConfigCompress ConfCompress;
	CConfigGeneral ConfGeneral;
	ConfCompress.load(ConfigManager);
	ConfGeneral.load(ConfigManager);

	CArchiverDLL *Archiver=ArchiverManager.GetArchiver(GetDllIDFromParameterType(Type));
	if(!Archiver || !Archiver->IsOK()){
		return false;
	}

	//UNICODEΉ̃`FbN
	if(!Archiver->IsUnicodeCapable()){		//UNICODEɑΉĂȂ
		if(!UtilCheckT2AList(ParamList)){
			//t@CUNICODEt@Ck悤Ƃ
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_UNICODEPATH)));
			return false;
		}
	}

	//----------------------------------------------
	// GZ/BZ2/JACKňk\(Pt@C)ǂ`FbN
	//----------------------------------------------
	if(PARAMETER_GZ==Type||PARAMETER_BZ2==Type||PARAMETER_JACK==Type){
		int nFileCount=0;
		CPath pathFileName;
		for(std::list<CString>::iterator ite=ParamList.begin();ite!=ParamList.end();++ite){
			nFileCount+=FindFileToCompress(*ite,pathFileName);
			if(nFileCount>=2){
				ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_SINGLE_FILE_ONLY)));
				return false;
			}
		}
		if(0==nFileCount){
			ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_FILE_NOT_SPECIFIED)));
			return false;
		}
		ParamList.clear();
		ParamList.push_back(pathFileName);
	}else if(PARAMETER_ISH==Type||PARAMETER_UUE==Type){	// ISH/UUEŕϊ\(t@Cǂ)`FbN
		for(std::list<CString>::iterator ite=ParamList.begin();ite!=ParamList.end();++ite){
			if(PathIsDirectory(*ite)){
				ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_CANT_COMPRESS_FOLDER)));
				return false;
			}
		}
	}

	//=========================================================
	// JgfBNgݒ̂߂\ŏI_pX擾
	//=========================================================
	CString pathBase;
	UtilGetBaseDirectory(pathBase,ParamList);
	TRACE(pathBase),TRACE(_T("\n"));

	//JgfBNgݒ
	SetCurrentDirectory(pathBase);

	//kt@C
	TRACE(_T("kt@C\n"));
	CPath pathArcFileName;
	CString strErr;
	HRESULT hr=GetArchiveName(pathArcFileName,ParamList,Type,Options,ConfCompress,ConfGeneral,lpszSpecificOutputFileName,lpszSpecificOutputDir,Archiver->IsUnicodeCapable(),strErr);
	if(FAILED(hr)){
		if(hr!=E_ABORT){
			ErrorMessage(strErr);
		}
		return false;
	}

	//====================================================================
	// ΉDLLŃt@CUNICODEt@Ck悤Ƃ
	//====================================================================
	if(!Archiver->IsUnicodeCapable()&&!UtilCheckT2A(pathArcFileName)){
		//GetArchiveNameŒe؂Ȃ(B2E/JACKȂ)Œe
		//B2E͕ʌnŏĂ邪AÔ߁B
		ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_UNICODEPATH)));
		return false;
	}

	if(PARAMETER_JACK!=Type){
		if(PathFileExists(pathArcFileName)){
			if(!DeleteFile(pathArcFileName)){
				//폜łȂ
				CString strLastError;
				UtilGetLastErrorMessage(strLastError);
				CString msg;
				msg.Format(IDS_ERROR_FILE_REPLACE,(LPCTSTR)strLastError);

				ErrorMessage(msg);
				return false;
			}
		}
//		if(Options&COMPRESS_SPLIT&&_T(".001")==pathArcFileName.Right(4)){	//k
//			pathArcFileName.Delete(pathArcFileName.GetLength()-4,4);
//		}

		//======================================
		// X|Xt@CɋL邽߂
		// kΏۃt@CC
		//======================================
		const int nBasePathLength=pathBase.GetLength();
		for(std::list<CString>::iterator ite=ParamList.begin();ite!=ParamList.end();++ite){
			//x[XpXɑ΃pX擾 : ʂłpX̕Jbg
			//(*ite)=(*ite).Right((*ite).GetLength()-BasePathLength);
			//(*ite).Delete(0,nBasePathLength);
			(*ite)=(LPCTSTR)(*ite)+nBasePathLength;

			//BasePathƓɂȂċɂȂĂ܂pX΁AJgfBNgwɓĂ
			if((*ite).IsEmpty()){
				*ite=_T(".");
			}
		}
	}

	//Z}tHɂr
	CSemaphoreLocker SemaphoreLock;
	if(ConfCompress.LimitCompressFileCount){
		SemaphoreLock.Lock(LHAFORGE_COMPRESS_SEMAPHORE_NAME,ConfCompress.MaxCompressFileCount);
	}


	TRACE(_T("ArchiverHandler Ăяo\n"));
	//------------
	// ks
	//------------
	CString strLog;
	/*
	format̎ẃAB2E32.dllł̂ݗL
	level̎ẃAB2E32.dllȊOŗL
	*/
	bool bRet=Archiver->Compress(pathArcFileName,ParamList,ConfigManager,Type,Options,lpszFormat,lpszMethod,lpszLevel,strLog);
	//---O\
	switch(ConfGeneral.LogViewEvent){
	case LOGVIEW_ON_ERROR:
		if(!bRet){
			CLogDialog LogDlg;
			LogDlg.SetData(strLog);
			LogDlg.DoModal();
		}
		break;
	case LOGVIEW_ALWAYS:
		CLogDialog LogDlg;
		LogDlg.SetData(strLog);
		LogDlg.DoModal();
		break;
	}

	//o͐tH_J
	if(bRet && ConfCompress.OpenDir){
		CPath pathOpenDir=pathArcFileName;
		pathOpenDir.RemoveFileSpec();
		pathOpenDir.AddBackslash();
		if(ConfGeneral.Filer.UseFiler){
			//p[^WJɕKvȏ
			std::map<stdString,CString> envInfo;
			MakeExpandInformationEx(envInfo,pathOpenDir,pathArcFileName);

			//R}hEp[^WJ
			CString strCmd,strParam;
			UtilExpandTemplateString(strCmd,ConfGeneral.Filer.FilerPath,envInfo);	//R}h
			UtilExpandTemplateString(strParam,ConfGeneral.Filer.Param,envInfo);	//p[^
			ShellExecute(NULL, _T("open"), strCmd,strParam, NULL, SW_SHOWNORMAL);
		}else{
			//ExplorerŊJ
			UtilNavigateDirectory(pathOpenDir);
		}
	}

	TRACE(_T("Exit Compress()\n"));
	return bRet;
}

void GetArchiveFileExtension(PARAMETER_TYPE type,LPCTSTR lpszOrgFile,CString &rExt,LPCTSTR lpszDefaultExt)
{
	switch(type){
	case PARAMETER_GZ:
	case PARAMETER_BZ2:
		rExt=PathFindExtension(lpszOrgFile);
		rExt+=lpszDefaultExt;
		break;
	case PARAMETER_JACK:
		//JACK̏ꍇɂ̓A[JCut@Co͐tH_ɂȂ
		//:t@CDLL
		rExt.Empty();
		break;
	default:
		rExt=lpszDefaultExt;
		break;
	}
}

HRESULT CheckArchiveName(LPCTSTR lpszArcFile,const std::list<CString> &rOrgFileList,bool bOverwrite,bool bUnicodeCapable,CString &strErr)
{
	// t@CƂ
	if(_tcslen(lpszArcFile)>=_MAX_PATH){
		strErr=CString(MAKEINTRESOURCE(IDS_ERROR_MAX_PATH));
		return E_LF_CANNOT_DECIDE_FILENAME;
	}

	// łt@C͌̃t@CƓ
	std::list<CString>::const_iterator ite,end;
	end=rOrgFileList.end();
	for(ite=rOrgFileList.begin();ite!=end;++ite){
		if(0==(*ite).CompareNoCase(lpszArcFile)){
			strErr.Format(IDS_ERROR_SAME_INPUT_AND_OUTPUT,lpszArcFile);
			return E_LF_OVERWRITE_SOURCE;
		}
	}

	// ΉDLLUNICODEt@CnĂȂ
	if(!bUnicodeCapable && !UtilCheckT2A(lpszArcFile)){
		strErr=CString(MAKEINTRESOURCE(IDS_ERROR_UNICODEPATH));
		return E_LF_CANNOT_DECIDE_FILENAME;
	}

	// t@Cɑ݂邩ǂ
	if(!bOverwrite){
		if(PathFileExists(lpszArcFile)){
			// t@C݂肠
			// ̏ꍇ̓G[bZ[WoɃt@C͂
			return S_LF_ARCHIVE_EXISTS;
		}
	}

	return S_OK;
}

int FindCompressionParamIndex(PARAMETER_TYPE type,int Options)
{
	//k`̏
	for(int nIndex=0;nIndex<COMPRESS_PARAM_COUNT;nIndex++){
		if(type==CompressParameterArray[nIndex].Type){
			if(Options==CompressParameterArray[nIndex].Options){
				return nIndex;
			}
		}
	}
	return -1;
}

//㏑mFȂ
HRESULT ConfirmOutputFile(CPath &r_pathArcFileName,std::list<CString> &rOrgFileNameList,LPCTSTR lpExt,BOOL bAskName,bool bUnicodeCapable)
{
	//----------------------------
	// t@Cw肪K؂mF
	//----------------------------
	bool bForceOverwrite=false;
	BOOL bInputFilenameFirst=bAskName;//Config.Common.Compress.SpecifyOutputFilename;

/*	if(bDriveRoot){		//hCu[gkꍇAt@CłȂ
		bInputFilenameFirst=TRUE;
	}*/

	HRESULT hStatus=E_UNEXPECTED;
	while(S_OK!=hStatus){
		if(!bInputFilenameFirst){
			//t@C̒ȂǊmF
			CString strLocalErr;
			hStatus=CheckArchiveName(r_pathArcFileName,rOrgFileNameList,bForceOverwrite,bUnicodeCapable,strLocalErr);
			if(FAILED(hStatus)){
				ErrorMessage(strLocalErr);
			}
			if(S_OK==hStatus){
				break;	//
			}
		}

		//ŏ̃t@Ĉ͉g
		//Iꂽt@CԂĂ
		CPath pathFile;
		if(E_LF_CANNOT_DECIDE_FILENAME==hStatus){
			//t@CłȂꍇAJgfBNgspXƂ
			TCHAR szBuffer[_MAX_PATH+1]={0};
			GetCurrentDirectory(_MAX_PATH,szBuffer);
			pathFile=szBuffer;
			pathFile.Append(CString(MAKEINTRESOURCE(IDS_UNTITLED)) + lpExt);
		}else{
			pathFile=r_pathArcFileName;
		}

		//==================
		// Otĕۑ
		//==================
		//tB^񐶐
		CString strFilter(MAKEINTRESOURCE(IDS_COMPRESS_FILE));
		strFilter.AppendFormat(_T(" (*%s)|*%s"),lpExt,lpExt);
		TCHAR filter[_MAX_PATH+2]={0};
		UtilMakeFilterString(strFilter,filter,COUNTOF(filter));

		CFileDialog dlg(FALSE, lpExt, pathFile, OFN_OVERWRITEPROMPT|OFN_NOCHANGEDIR,filter);
		if(IDCANCEL==dlg.DoModal()){	//LZ
			return E_ABORT;
		}

		r_pathArcFileName=dlg.m_szFileName;
		//DELETED:t@C̃OpX擾

		bForceOverwrite=true;
		bInputFilenameFirst=false;
	}
	return S_OK;
}

/*************************************************************
A[JCut@C肷B

 {Iɂ́AR}hCŗ^t@ĈA
ŏ̃t@CɂāAgqςt@CB
*************************************************************/
HRESULT GetArchiveName(CPath &r_pathArcFileName,std::list<CString> &OrgFileNameList,const PARAMETER_TYPE Type,const int Options,const CConfigCompress &ConfCompress,const CConfigGeneral &ConfGeneral,LPCTSTR lpszSpecificOutputFileName,LPCTSTR lpszSpecificOutputDir,bool bUnicodeCapable,CString &strErr)
{
	ASSERT(!OrgFileNameList.empty());

	//==================
	// t@C̐
	//==================
	BOOL bDirectory=FALSE;	//kΏۂɎw肳ꂽtH_Ȃtrue;tH_ƃt@CŖ'.'̈ς
	BOOL bDriveRoot=FALSE;	//kΏۂhCũ[g(X:\ etc.)Ȃtrue
	CPath pathOrgFileName;

	bool bFileNameSpecified=(NULL!=lpszSpecificOutputFileName && 0<_tcslen(lpszSpecificOutputFileName));

	//---̃t@C
	if(bFileNameSpecified){
		//TODO:ŏ̂͂
		pathOrgFileName=lpszSpecificOutputFileName;
		pathOrgFileName.StripPath();	//pX菜
	}else{
		//擪̃t@CɎbIɃt@Co
		pathOrgFileName=(LPCTSTR)*OrgFileNameList.begin();

		//---
		//fBNgǂ
		bDirectory=pathOrgFileName.IsDirectory();
		//hCũ[gǂ
		CPath pathRoot=pathOrgFileName;
		pathRoot.AddBackslash();
		bDriveRoot=pathRoot.IsRoot();
		if(bDriveRoot){
			//{[擾
			TCHAR szVolume[MAX_PATH];
			GetVolumeInformation(pathRoot,szVolume,MAX_PATH,NULL,NULL,NULL,NULL,0);
			//hCuۂƈkꍇɂ́A{[t@CƂ
			UtilFixFileName(pathOrgFileName,szVolume,_T('_'));
		}
	}

	//k`̏
	int nIndex=FindCompressionParamIndex(Type,Options);
	if(-1==nIndex){
		strErr=CString(MAKEINTRESOURCE(IDS_ERROR_ILLEGAL_FORMAT_TYPE));
		return E_FAIL;
	}

	//gq
	CString strExt;
	if(bFileNameSpecified){
		strExt=PathFindExtension(pathOrgFileName);
	}else{
		GetArchiveFileExtension(Type,pathOrgFileName,strExt,CompressParameterArray[nIndex].Ext);
	}
//	if(Options&COMPRESS_SPLIT){
//		Ext+=_T(".001");
//	}

	//o͐tH_
	CPath pathOutputDir;
	if(NULL!=lpszSpecificOutputDir && 0!=_tcslen(lpszSpecificOutputDir)){
		//o͐tH_w肳Ăꍇ
		pathOutputDir=lpszSpecificOutputDir;
	}else{
		//ݒɏo͐߂
		bool bTmp;	//TODO
		HRESULT hr=GetOutputDirPathFromConfig(ConfCompress.OutputDirType,pathOrgFileName,ConfCompress.OutputDir,pathOutputDir,bTmp,strErr);
		if(FAILED(hr)){
			return hr;
		}
	}
	pathOutputDir.AddBackslash();

	// o͐悪lbg[NhCu/[oufBXNłȂx
	// o͐悪݂ȂȂA쐬mF
	HRESULT hRes=ConfirmOutputDir(ConfGeneral,pathOutputDir,strErr);
	if(FAILED(hRes)){
		//LZȂ
		return hRes;
	}

	if(PARAMETER_JACK==Type){
		//JACK`̏ꍇ͏o̓tH_w
		r_pathArcFileName=pathOutputDir;
		return S_OK;
	}

	//t@CgqƃpX폜
	if(bDirectory || bDriveRoot){	//fBNg
		//fBNg͊gq(Ō'.'ȍ~̕)폜Ȃ
		CPath pathDir=pathOrgFileName;
		pathDir.StripPath();

		r_pathArcFileName=pathOutputDir;
		r_pathArcFileName.Append(pathDir);
		r_pathArcFileName=(LPCTSTR)(((CString)r_pathArcFileName)+strExt);
	}else{	//t@C
		CPath pathFile=pathOrgFileName;
		pathFile.StripPath();
		r_pathArcFileName=pathOutputDir;
		r_pathArcFileName.Append(pathFile);
		if(PARAMETER_B2E!=Type){
			//B2E̎͊gq̈B2E32.dllɔC
			r_pathArcFileName.RemoveExtension();
		}
		r_pathArcFileName=(LPCTSTR)(((CString)r_pathArcFileName)+strExt);
	}

	//---B2Eʈ
	if(PARAMETER_B2E==Type){
		return S_OK;
	}

	//hCu[gkꍇAt@C͑Ȃ
	return ConfirmOutputFile(r_pathArcFileName,OrgFileNameList,strExt,ConfCompress.SpecifyOutputFilename || bDriveRoot,bUnicodeCapable);
}


//pathFileName͍ŏ̃t@C
//[DT
int FindFileToCompress(LPCTSTR lpszPath,CPath &pathFileName)
{
	if(!PathIsDirectory(lpszPath)){	//pXt@C̏ꍇ
		pathFileName=lpszPath;
		return 1;
	}else{	//pXtH_̏ꍇAtH_̃t@CT
		int nFileCount=0;

		CPath pathWild=lpszPath;
		pathWild.Append(_T("\\*"));
		CFindFile cFind;

		BOOL bFound=cFind.FindFile(pathWild);
		for(;bFound;bFound=cFind.FindNextFile()){
			if(cFind.IsDots())continue;

			if(cFind.IsDirectory()){	//TufBNg
				nFileCount+=FindFileToCompress(cFind.GetFilePath(),pathFileName);
				if(nFileCount>=2){
					return 2;
				}
			}else{
				pathFileName=(LPCTSTR)cFind.GetFilePath();
				nFileCount++;
				if(nFileCount>=2){
					return 2;
				}
			}
		}
		return nFileCount;
	}
}

